#-----------------------------------------------------------------------------
#
#   Supersymmetry - Pipe buying dialog
#
#-----------------------------------------------------------------------------

from pygame import Rect
from albow.dialogs import Dialog
from albow.palette_view import PaletteView
from albow.resource import get_font
from albow.utils import blit_in_rect
from tiles import buyable_pipes
from constants import *

#-----------------------------------------------------------------------------

class PipePalette(PaletteView):

	margin = 8
	font = get_font(12, "Vera.ttf")

	def __init__(self, world):
		self.world = world
		cs = (TILE_SIZE + 20, TILE_SIZE + 20)
		PaletteView.__init__(self, cs, 1, 4)
	
	def num_items(self):
		return len(buyable_pipes)
	
	def draw_item(self, surf, i, rect):
		r = Rect(0, 0, TILE_SIZE, TILE_SIZE)
		r.midtop = rect.midtop
		tile = buyable_pipes[i]
		blit_in_rect(surf, tile.image, r, 'c')
		cost = self.cost_of_tile(tile)
		buf = self.font.render("{:,d}m".format(cost // 1000000), True, self.fg_color)
		blit_in_rect(surf, buf, rect, 'b')
	
	def click_item(self, i, e):
		self.buy_item(i)
	
	def item_is_selected(self, i):
		return False
	
	def cost_of_tile(self, tile):
		return int(tile.cost * ORDER_MULTIPLIER)
	
	def buy_item(self, i):
		tile = buyable_pipes[i]
		world = self.world
		cost = self.cost_of_tile(tile)
		if world.spend_funds_with_sound(cost):
			world.add_tile_to_inventory(tile)
		
#-----------------------------------------------------------------------------

class BuyPipesDialog(Dialog):

	def __init__(self, world):
		self.world = world
		self.palette = PipePalette(world)
		Dialog.__init__(self, client = self.palette, responses = ["Done"])
