#-----------------------------------------------------------------------------
#
#   Supersymmetry - Playing screen
#
#-----------------------------------------------------------------------------

from pygame import draw, Color
from pygame.locals import *
from albow.widget import Widget
from albow.controls import Label, Button, ValueDisplay
from albow.palette_view import PaletteView
from albow.layout import Row, Column, Grid
from albow.references import Ref
from albow.resource import get_font, get_image
from albow.image_array import get_image_array
from humerus.main_commands import MainCommandScreen
#from grid_view import GridView
from game import game
from constants import *
from tiles import tile_list
from buy_pipes import BuyPipesDialog
from funding import FundingDialog
from game import game
import tiles

#-----------------------------------------------------------------------------

class GridView(Widget):
	#  game     Game
	#  client   PlayScreen
	
	bg_image = get_image("europe.png")
	
	@property
	def world(self):
		return self.game.world
	
	def __init__(self, game, client, **kwds):
		Widget.__init__(self, **kwds)
		self.game = game
		self.client = client

	def draw(self, surf):
		world = self.world
		grid = world.grid
		x0, y0 = self.origin()
		self.draw_grid_lines(surf, world, x0, y0)
		b = world.grid_bounds
		for row in xrange(b.top, b.bottom):
			y = y0 + row * TILE_SIZE
			for col in xrange(b.left, b.right):
				x = x0 + col * TILE_SIZE
				tile = tile_list[grid[row][col]]
				if tile:
					tile.draw(surf, x, y)
	
	def draw_grid_lines(self, surf, world, x0, y0):
		s = TILE_SIZE
		c = Color(0, 0, 255)
		b = world.grid_bounds
		x1 = x0 + s * b.left
		y1 = y0 + s * b.top
		x2 = x0 + s * b.right
		y2 = y0 + s * b.bottom
		for i in xrange(b.top, b.bottom + 1):
			y = y0 + s * i
			draw.line(surf, c, (x1, y), (x2, y))
		for i in xrange(b.left, b.right + 1):
			x = x0 + s * i
			draw.line(surf, c, (x, y1), (x, y2))
	
	def origin(self):
		w = GRID_COLS * TILE_SIZE
		h = GRID_ROWS * TILE_SIZE
		x0 = (self.width - w) // 2
		y0 = (self.height - h) // 2
		return (x0, y0)		
	
	def mouse_down(self, e):
		x0, y0 = self.origin()
		x, y = e.local
		col = (x - x0) // TILE_SIZE
		row = (y - y0) // TILE_SIZE
		cell = (col, row)
		world = game.world
		if world.cell_in_bounds(cell):
			if e.shift:
				world.remove_tile(cell)
			else:
				world.place_tile(cell)

#-----------------------------------------------------------------------------

#class InventoryView(Widget):
#
#	margin = 2
#	sel_color = (0, 128, 255, 128)
#
#	def __init__(self, **kwds):
#		Widget.__init__(self, **kwds)
#		self.height = 2 * self.margin + TILE_SIZE
#	
#	def draw(self, surf):
#		x = y = self.margin
#		dx = self.margin + TILE_SIZE
#		sel = game.world.inventory_selection
#		for i, tileno in enumerate(game.world.inventory):
#			if i == sel:
#				draw.rect(surf, self.sel_color, (x, y, TILE_SIZE, TILE_SIZE))
#			tiles.get(tileno).draw(surf, x, y)
#			x += dx
#	
#	def mouse_down(self, e):
#		x, y = e.local
#		i = (x - self.margin) // (TILE_SIZE + self.margin)
#		if 0 <= i < len(game.world.inventory):
#			game.world.inventory_selection = i

class InventoryView(PaletteView):

	margin = 2
	sel_color = (0, 128, 255, 128)
	highlight_style = 'fill'
	
	def __init__(self, ncols, **kwds):
		cs = TILE_SIZE + self.margin
		PaletteView.__init__(self, (cs, cs), 3, ncols, scrolling = True, **kwds)

	def num_items(self):
		return len(game.world.inventory)
		
	def draw_item(self, surf, i, rect):
		tileno = game.world.inventory[i]
		tiles.get(tileno).draw_centered(surf, rect)
	
	def click_item(self, i, e):
		world = game.world
		if 0 <= i < len(world.inventory):
			world.inventory_selection = i
		
	def item_is_selected(self, i):
		return game.world.inventory_selection == i

#-----------------------------------------------------------------------------

class FundsDisplay(ValueDisplay):

	def format_value(self, x):
		return "{:,d}".format(x)
	
#-----------------------------------------------------------------------------

class UIBar(Row):

	bg_image = get_image("ui_background.png")

#-----------------------------------------------------------------------------

class CmdButton(Button):

	font = get_font(12, "Vera.ttf")
	
#	def get_enabled(self):
#		result = Button.get_enabled(self)
#		print "CmdButton('%s').get_enabled: %s" % (self.text, result)
#		return result

#-----------------------------------------------------------------------------

class PlayScreen(MainCommandScreen):

	ui_height = 132
	
	@property
	def world(self):
		return game.world
	
	def __init__(self, shell):
		MainCommandScreen.__init__(self, shell)
		self.grid_view = GridView(shell.game, self)
		self.grid_view.size = (self.width, self.height - self.ui_height)
		self.add(self.grid_view)
		self.ui_bar = self.create_ui_bar()
		self.ui_bar.bottomleft = (0, self.height)
		self.add(self.ui_bar)

	def create_ui_bar(self):
		inv_view = self.create_inventory_view()
		#inv_view.width = self.width
		stat_view = self.create_status_view()
		stat_view.right = self.width
		buttons = self.create_buttons()
		buttons.topright = stat_view.topleft
		ui_bar = UIBar([inv_view, buttons, stat_view],
			expand = inv_view, margin = 4, spacing = 4, align = 't')
		ui_bar.set(width = self.width, height = self.ui_height)
#		ui_bar.add(inv_view)
#		ui_bar.add(stat_view)
#		ui_bar.add(buttons)
		return ui_bar
	
	def create_inventory_view(self):
		view = InventoryView(6)
		return view
	
	def create_status_view(self):
		wref = Ref(game).world
		return Grid([
			[Label("Project Phase"), ValueDisplay(ref = wref.level_number)],
			[Label("Funds"), FundsDisplay(ref = wref.funds, width = "999,999,999,999")],
			[Label("Beam Path"), ValueDisplay(ref = wref.beam_status)],
			[Label("Vacuum OK"), ValueDisplay(ref = wref.vacuum_status)],
			[Label("Beam Energy"), ValueDisplay(ref = Ref(self).beam_energy_status, width = "999 TeV   (-999)")],
			[Label("Required Energy"), ValueDisplay(ref = wref.energy_required, format = "%s TeV")],
		], margin = 4, bg_color = Color("white"), border_width = 2, row_spacing = 2)
	
	@property
	def beam_energy_status(self):
		world = game.world
		e = world.beam_energy
		return "%s TeV   (%+d)" % (e, e - world.energy_required)
	
	def create_buttons(self):
		def column(items):
			return Column(items, spacing = 4, equalize = 'w')
		wref = Ref(game).world
		col1 = column([
			CmdButton("Buy Random Pipe", action = +wref.buy_random_pipe),
			CmdButton("Buy Selected Pipes", action = self.buy_pipes),
			CmdButton("Apply for Funds", action = self.apply_for_funds, enable = +wref.extra_funds_available),
		])
		col2 = column([
			CmdButton("Start Beam", action = +wref.start, enable = +wref.is_startable),
			CmdButton("Stop Beam", action = +wref.stop, enable = +wref.is_running),
			CmdButton("Restart Level", action = +wref.restart_level),
		])
		return Row([col1, col2], align = 't',
			margin = 6, spacing = 4, border_width = 2, bg_color = Color("white"), equalize = 'w')

	def buy_pipes(self):
		dlog = BuyPipesDialog(game.world)
		dlog.present()

	def apply_for_funds(self):
		dlog = FundingDialog(game.world)
		dlog.present()
